USE AdventureWorks2016CTP3;
SET NOCOUNT ON;
SET ANSI_NULLS ON;
SET QUOTED_IDENTIFIER ON;
GO

IF NOT EXISTS (SELECT * FROM sys.tables WHERE tables.name = 'database_mail_failure')
BEGIN
	CREATE TABLE dbo.database_mail_failure
	(	database_mail_failure_id INT IDENTITY(1,1) NOT NULL CONSTRAINT PK_database_mail_failure PRIMARY KEY CLUSTERED,
		error_time_utc DATETIME NOT NULL,
		error_time_local DATETIME NOT NULL,
		error_description VARCHAR(MAX) NULL,
		mail_item_id INT NOT NULL,
		mail_profile_name VARCHAR(128) NOT NULL,
		mail_recipients VARCHAR(MAX) NOT NULL,
		mail_recipients_cc VARCHAR(MAX) NULL,
		mail_recipients_bcc VARCHAR(MAX) NULL,
		mail_subject VARCHAR(MAX) NULL,
		mail_body VARCHAR(MAX) NULL,
		mail_body_format VARCHAR(20) NULL,
		mail_importance VARCHAR(6) NULL,
		mail_sensitivity VARCHAR(12) NULL,
		file_attachments VARCHAR(MAX) NULL,
		attachment_encoding VARCHAR(20) NULL,
		query VARCHAR(MAX) NULL,
		query_database VARCHAR(100) NULL,
		attach_query_result_as_file BIT NULL,
		query_result_header BIT NOT NULL,
		query_result_width INT,
		query_result_separator VARCHAR(1),
		exclude_query_output BIT,
		append_query_error BIT,
		mail_send_request_added_to_queue_time_utc DATETIME NOT NULL,
		mail_send_request_user VARCHAR(100) NOT NULL,
		mail_send_request_removed_from_queue_time_utc DATETIME NULL,
		has_email_been_sent_to_operator BIT NOT NULL);

	CREATE NONCLUSTERED INDEX IX_database_mail_failure_error_time_utc ON dbo.database_mail_failure (error_time_utc);
	CREATE NONCLUSTERED INDEX IX_database_mail_failure_error_time_local ON dbo.database_mail_failure (error_time_local);
END
GO

IF EXISTS (SELECT * FROM sys.procedures WHERE procedures.name = 'monitor_database_mail_failures')
BEGIN
	DROP PROCEDURE monitor_database_mail_failures;
END
GO

CREATE PROCEDURE dbo.monitor_database_mail_failures
	@minutes_to_monitor SMALLINT = 1440
AS
BEGIN
	SET NOCOUNT ON;
	-- Determine UTC offset so that all times can easily be converted to UTC.
	DECLARE @utc_offset INT;
	SELECT
		@utc_offset = -1 * DATEDIFF(HOUR, GETUTCDATE(), GETDATE());

	-- Find all recent database mail failures and log them in the target log table.
	INSERT INTO dbo.database_mail_failure
		(error_time_utc, error_time_local, error_description, mail_item_id, mail_profile_name, mail_recipients, mail_recipients_cc, mail_recipients_bcc,
		 mail_subject, mail_body, mail_body_format, mail_importance, mail_sensitivity, file_attachments, attachment_encoding,
		 query, query_database, attach_query_result_as_file, query_result_header, query_result_width, query_result_separator,
		 exclude_query_output, append_query_error, mail_send_request_added_to_queue_time_utc, mail_send_request_user,
		 mail_send_request_removed_from_queue_time_utc, has_email_been_sent_to_operator)
	SELECT DISTINCT
			DATEADD(HOUR, @utc_offset, sysmail_faileditems.last_mod_date) AS error_time_utc,
			sysmail_faileditems.last_mod_date AS error_time_local,
			REPLACE(REPLACE(sysmail_event_log.description, CHAR(10), ' '), CHAR(13), ' ') AS error_description,
			sysmail_faileditems.mailitem_id AS mail_item_id,
			sysmail_profile.name AS mail_profile_name,
			sysmail_faileditems.recipients AS mail_recipients,
			sysmail_faileditems.copy_recipients AS mail_recipients_cc,
			sysmail_faileditems.blind_copy_recipients AS mail_recipients_bcc,
			sysmail_faileditems.subject AS mail_subject,
			sysmail_faileditems.body AS mail_body,
			sysmail_faileditems.body_format AS mail_body_format,
			sysmail_faileditems.importance AS mail_importance,
			sysmail_faileditems.sensitivity AS mail_sensitivity,
			sysmail_faileditems.file_attachments,
			sysmail_faileditems.Attachment_encoding AS attachment_encoding,
			sysmail_faileditems.Query AS query,
			sysmail_faileditems.execute_query_database AS query_database,
			sysmail_faileditems.attach_query_result_as_file,
			sysmail_faileditems.query_result_header,
			sysmail_faileditems.query_result_width,
			sysmail_faileditems.query_result_separator,
			sysmail_faileditems.exclude_query_output,
			sysmail_faileditems.append_query_error,
			DATEADD(HOUR, @utc_offset, sysmail_faileditems.send_request_date) AS mail_send_request_added_to_queue_time_utc,
			sysmail_faileditems.send_request_user AS mail_send_request_user,
			DATEADD(HOUR, @utc_offset, sysmail_faileditems.sent_date) AS mail_send_request_removed_from_queue_time_utc,
			0 AS has_email_been_sent_to_operator
	FROM msdb.dbo.sysmail_faileditems
	LEFT JOIN msdb.dbo.sysmail_event_log
	ON sysmail_faileditems.mailitem_id = sysmail_event_log.mailitem_id
	LEFT JOIN msdb.dbo.sysmail_profile
	ON sysmail_profile.profile_id = sysmail_faileditems.profile_id
	WHERE DATEADD(HOUR, @utc_offset, sysmail_faileditems.send_request_date) > DATEADD(MINUTE, -1 * @minutes_to_monitor, GETUTCDATE())
	AND sysmail_faileditems.mailitem_id NOT IN (
		SELECT database_mail_failure.mail_item_id FROM dbo.database_mail_failure WHERE database_mail_failure.mail_send_request_added_to_queue_time_utc > DATEADD(MINUTE, -1 * @minutes_to_monitor, GETUTCDATE()))

	DECLARE @profile_name VARCHAR(MAX) = 'Default Public Profile';
	DECLARE @email_to_address VARCHAR(MAX) = 'epollack@mydomain.com';
	DECLARE @email_subject VARCHAR(MAX);
	DECLARE @email_body VARCHAR(MAX);

	IF EXISTS (SELECT * FROM dbo.database_mail_failure WHERE database_mail_failure.has_email_been_sent_to_operator = 0)
	BEGIN
		SELECT @email_subject = 'Failed Database Mail Alert: ' + ISNULL(@@SERVERNAME, CAST(SERVERPROPERTY('ServerName') AS VARCHAR(MAX)));

		SELECT @email_body = 'At least one database mail failure has occurred on ' + ISNULL(@@SERVERNAME, CAST(SERVERPROPERTY('ServerName') AS VARCHAR(MAX))) + ':
	<html><body><table border=1>
	<tr>
		<th bgcolor="#F29C89">Server Error Time</th>
		<th bgcolor="#F29C89">Error Description</th>
		<th bgcolor="#F29C89">Mail Recipients</th>
		<th bgcolor="#F29C89">Mail Subject</th>
		<th bgcolor="#F29C89">Mail Body Format</th>
		<th bgcolor="#F29C89">Mail Attachments</th>
		<th bgcolor="#F29C89">Query</th>
		<th bgcolor="#F29C89">Query Database</th>
	</tr>';
		SELECT @email_body = @email_body + CAST((SELECT CAST(DATEADD(HOUR, -1 * @utc_offset, database_mail_failure.error_time_utc) AS VARCHAR(MAX)) AS 'td', '',
										  database_mail_failure.error_description AS 'td', '',
										  database_mail_failure.mail_recipients AS 'td', '',
										  database_mail_failure.mail_subject AS 'td',
										  database_mail_failure.mail_body_format AS 'td', '',
										  database_mail_failure.file_attachments AS 'td', '',
										  database_mail_failure.query AS 'td',
										  database_mail_failure.query_database AS 'td'
		FROM dbo.database_mail_failure
		WHERE database_mail_failure.has_email_been_sent_to_operator = 0
		ORDER BY database_mail_failure.error_time_utc ASC
		FOR XML PATH('tr'), ELEMENTS) AS VARCHAR(MAX));

		SELECT @email_body = @email_body + '</table></body></html>';
		SELECT @email_body = REPLACE(@email_body, '<td>', '<td valign="top">');

		EXEC msdb.dbo.sp_send_dbmail
			@profile_name = @profile_name,
			@recipients = @email_to_address,
			@subject = @email_subject,
			@body_format = 'html',
			@body = @email_body;

		UPDATE database_mail_failure
			SET has_email_been_sent_to_operator = 1
		FROM dbo.database_mail_failure
		WHERE database_mail_failure.has_email_been_sent_to_operator = 0;
	END
END
GO

IF EXISTS (SELECT * FROM sys.views WHERE views.name = 'v_failed_database_mail_message_detail')
BEGIN
	DROP VIEW dbo.v_failed_database_mail_message_detail;
END
GO

CREATE VIEW dbo.v_failed_database_mail_message_detail
AS
	WITH CTE_DATABASE_MAIL_QUERY AS (
		SELECT DISTINCT
			CAST('EXEC msdb.dbo.Sp_send_dbmail
				@profile_name = ''' + CAST(database_mail_failure.mail_profile_name AS NVARCHAR(MAX)) + ''',' +
				CASE WHEN database_mail_failure.mail_profile_name IS NOT NULL THEN '
					@recipients = ''' + CAST(database_mail_failure.mail_recipients AS NVARCHAR(MAX))  + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.mail_recipients_cc IS NOT NULL THEN '
					@copy_recipients = ''' + CAST(database_mail_failure.mail_recipients_cc AS NVARCHAR(MAX))  + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.mail_recipients_bcc IS NOT NULL THEN '
					@blind_copy_recipients = ''' + CAST(database_mail_failure.mail_recipients_bcc AS NVARCHAR(MAX))  + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.mail_body IS NOT NULL THEN '
					@body = ''' + CAST(REPLACE(database_mail_failure.mail_body, '''', '''''') AS NVARCHAR(MAX))  + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.mail_subject IS NOT NULL THEN '
					@subject = ''' + CAST(REPLACE(database_mail_failure.mail_subject, '''', '''''') AS NVARCHAR(MAX))  + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.mail_body_format IS NOT NULL THEN '
					@body_format = ''' + CAST(database_mail_failure.mail_body_format AS NVARCHAR(MAX)) + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.mail_importance IS NOT NULL THEN '
					@importance = ''' + CAST(database_mail_failure.mail_importance AS NVARCHAR(MAX)) + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.mail_sensitivity IS NOT NULL THEN '
					@sensitivity = ''' + CAST(database_mail_failure.mail_sensitivity AS NVARCHAR(MAX)) + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.file_attachments IS NOT NULL THEN '
					@file_attachments = ''' + CAST(REPLACE(database_mail_failure.file_attachments, '''', '''''') AS NVARCHAR(MAX)) + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.query IS NOT NULL THEN '
					@query = ''' + CAST(REPLACE(database_mail_failure.query, '''', '''''') AS NVARCHAR(MAX)) + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.query_database IS NOT NULL THEN '
					@execute_query_database = ''' + CAST(database_mail_failure.query_database AS NVARCHAR(MAX)) + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.attach_query_result_as_file IS NOT NULL THEN '
					@attach_query_result_as_file = ' + CAST(database_mail_failure.attach_query_result_as_file AS NVARCHAR(MAX)) + ',' ELSE '' END +
				CASE WHEN database_mail_failure.query_result_header IS NOT NULL THEN '
					@query_result_header = ' + CAST(database_mail_failure.query_result_header AS NVARCHAR(MAX)) + ',' ELSE '' END +
				CASE WHEN database_mail_failure.query_result_width IS NOT NULL THEN '
					@query_result_width  = ' + CAST(database_mail_failure.query_result_width AS NVARCHAR(MAX)) + ',' ELSE '' END +
				CASE WHEN database_mail_failure.query_result_separator IS NOT NULL THEN '
					@query_result_separator  = ''' + CAST(database_mail_failure.query_result_separator AS NVARCHAR(MAX)) + ''',' ELSE '' END +
				CASE WHEN database_mail_failure.exclude_query_output IS NOT NULL THEN '
					@exclude_query_output  = ' + CAST(database_mail_failure.exclude_query_output AS NVARCHAR(MAX)) + ',' ELSE '' END +
				CASE WHEN database_mail_failure.append_query_error IS NOT NULL THEN '
					@append_query_error  = ' + CAST(database_mail_failure.append_query_error AS NVARCHAR(MAX)) + ',' ELSE '' END AS NVARCHAR(MAX)) AS database_mail_query,
				database_mail_failure.*
		FROM dbo.database_mail_failure)
	SELECT
		CASE
			WHEN RIGHT(CTE_DATABASE_MAIL_QUERY.Database_Mail_Query, 1) = ','
				THEN SUBSTRING(CTE_DATABASE_MAIL_QUERY.Database_Mail_Query, 1, LEN(CTE_DATABASE_MAIL_QUERY.Database_Mail_Query) - 1)
			ELSE CTE_DATABASE_MAIL_QUERY.Database_Mail_Query
		END AS database_mail_query,
		CTE_DATABASE_MAIL_QUERY.mail_item_id,
		CTE_DATABASE_MAIL_QUERY.mail_profile_name,
		CTE_DATABASE_MAIL_QUERY.mail_recipients,
		CTE_DATABASE_MAIL_QUERY.mail_recipients_cc,
		CTE_DATABASE_MAIL_QUERY.mail_recipients_bcc,
		CTE_DATABASE_MAIL_QUERY.mail_subject,
		CTE_DATABASE_MAIL_QUERY.mail_body,
		CTE_DATABASE_MAIL_QUERY.mail_body_format,
		CTE_DATABASE_MAIL_QUERY.mail_importance,
		CTE_DATABASE_MAIL_QUERY.mail_sensitivity,
		CTE_DATABASE_MAIL_QUERY.file_attachments,
		CTE_DATABASE_MAIL_QUERY.attachment_encoding,
		CTE_DATABASE_MAIL_QUERY.query,
		CTE_DATABASE_MAIL_QUERY.query_database,
		CTE_DATABASE_MAIL_QUERY.attach_query_result_as_file,
		CTE_DATABASE_MAIL_QUERY.query_result_header,
		CTE_DATABASE_MAIL_QUERY.query_result_width,
		CTE_DATABASE_MAIL_QUERY.query_result_separator,
		CTE_DATABASE_MAIL_QUERY.exclude_query_output,
		CTE_DATABASE_MAIL_QUERY.append_query_error,
		CTE_DATABASE_MAIL_QUERY.mail_send_request_added_to_queue_time_utc,
		CTE_DATABASE_MAIL_QUERY.mail_send_request_user,
		CTE_DATABASE_MAIL_QUERY.mail_send_request_removed_from_queue_time_utc,
		CTE_DATABASE_MAIL_QUERY.error_description,
		CTE_DATABASE_MAIL_QUERY.error_time_local,
		CTE_DATABASE_MAIL_QUERY.error_time_utc
	FROM CTE_DATABASE_MAIL_QUERY;
GO
